#ifndef ASTAR_H
#define ASTAR_H

/// \author Randy Ribler
/// \date Oct, 2004

#include <queue>
#include <set>
#include <string>
using namespace std;

#include "DerefGreater.h"

class AStarNode;
class AStar
{
   public:
      AStar();
      int search(const AStarNode* root, 
		 const AStarNode* goal, vector<const AStarNode*>& result);
      void addToOpen(AStarNode* newNode);
      virtual ~AStar();
      
   private:
      // The open list is implemented as a priority queue.  We use the
      // DerefGreater functor to compare two AStar pointers.  It
      // dereferences the pointer so that it actually compares the objects
      // that the pointers point to.
      priority_queue<AStarNode*, vector<AStarNode*>, DerefGreater<AStarNode> > 
      openList_;

      // The seenList keeps track of the nodes that we have already
      // encountered.
      set<string> seenList_;

      // Closed list is maintained so that we don't destroy the parent
      // nodes contained in the solution.
      queue<AStarNode*> closedList_;
};

#endif
